#include "kweather.h"
#include "weather.h"
#include <iostream.h>
#include <kwm.h>
#include <qtooltip.h>
#include <kpopmenu.h>
#include <kfm.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <unistd.h>

Kweather::Kweather(QWidget*parent,const char* name):QWidget(parent,name){
	// Here we should load the conf file.
  loadConf();
  // Get our PID so we can generate a uniqe file name.
	int iPid = getpid();
	pid.setNum( iPid );

	kfmConn = new KFM();

	dispTemp = new weatherDisplay( this, "tempDisp");
	dispTemp->setAutoResize (true);
	dispTemp->setText("???\n???");
	
	timeOut = new QTimer(this, "timeOut" );
	timeOut->changeInterval(minutes*60000);

	QToolTip::add( this, "Current Weather:" );
	connect(timeOut, SIGNAL( timeout()), this, SLOT( loadFile()));
	connect(kfmConn, SIGNAL( finished()), this, SLOT( processData()));
	loadFile();
}

void Kweather::loadFile()
{
  if ( !offline )
	{
		QString dataDir = "file:/tmp/data.txt.";
		dataDir += pid;
		//cout << "Loading File " << endl;
		timeOut->stop();
		QString url = "ftp://weather.noaa.gov/data/observations/metar/stations/";
		url += stationId;
		url += ".TXT";
		kfmConn->copy(url,dataDir);
	}
}

void Kweather::processData()
{
	// files down lets get it!
	// we will expect the file to be named the DATA.TXT in the ~/.kde/apps/kweather/
	//cout << "Data is done..." << endl;
	QString dataDir = "/tmp/data.txt.";
	dataDir += pid;
	metarData = "";
	QFile f(dataDir);
if ( f.open(IO_ReadOnly) ) {    // file opened successfully
     QTextStream t( &f );        // use a text stream

     while ( !t.eof() ) {        // until end of file...
       	metarData += t.readLine();       // line of text excluding '\n'     	
       	}
     	f.close();

	currentWeather.getNewData( metarData );
	//cout << "Wind Speed: " << currentWeather.getWindSpeed() << endl;
	//cout << "Wind Direction: " << currentWeather.getWindDir() << endl;
	//cout << "Visability: " << currentWeather.getVisability() << endl;
	//cout << "Temprature: " << weather::toUS(usa, currentWeather.getTemp()) << endl;
	//cout << "Dewpoint: " << weather::toUS(usa, currentWeather.getDewPoint()) << endl;
	//cout << "Current Weather: " << currentWeather.getCurrentWeather() << endl;
	//cout << "Cloud Cover: " << currentWeather.getCover() << endl;
	QString tempTemp;
	QString tempTwo;
	int	tf;
	
//	tempTemp.setNum( (int)weather::toUS(usa, currentWeather.getTemp()));
	tf = currentWeather.getTemp();
	if (tf == -100) /* temperature not available */
	    tempTemp = "N/A\n";
	else    
	{
	tempTemp.setNum((int)tf);
	if (usa) tempTemp +="\260F\n";
	else tempTemp +="\260C\n";
	}
	
//	tempTwo.setNum( currentWeather.getWindSpeed());
	tf = currentWeather.getWindSpeed();
	if (!usa) tf = (tf * 185)/100;
	tempTwo.setNum((int)tf);
	tempTemp +=tempTwo;
	tempTemp += currentWeather.getWindDir();
	dispTemp->setText( tempTemp );

	QToolTip::add( this, tempTemp );
	QFile::remove(dataDir);
	}

	timeOut->start(minutes*60000);
}
Kweather::~Kweather(){

}


void Kweather::exitApp()
{
  	// Here i would like to save the conf file...
	QString configDir = KApplication:: localkdedir();
	QString configData;
	QString tempStr;
	configDir += "/share/config/kweatherrc";
	 // Parse Config Data
	 // Data will be in the following format
	 // :usa:online:stationid:timeout:
	 // :0:0:ABCD:000:
	configData += ":";
	tempStr.setNum(usa);
	configData += tempStr;
	configData += ":";
	tempStr.setNum(offline);
	configData += tempStr;
	configData += ":";
	configData += stationId;
	configData += ":";
	tempStr.setNum(minutes);
	configData += tempStr;
	configData += "\n";
	//cout << configData << endl;

 	QFile f(configDir);
	if ( f.open(IO_WriteOnly) )
	{    // file opened successfully
        f.writeBlock( configData, strlen(configData) );       // write to stderr
     		f.close();
	 }
	KApplication::exit(0);
}

void Kweather::configure()
{
	cfgView = new configView(this, "config");
	cfgView->setTimeout( minutes);
 	cfgView->setStation( stationId );
	cfgView->setUsa( usa );
	cfgView->setOnline( offline );
	cfgView->show();
}

void Kweather::change()
{
 minutes = cfgView->getTimeout();
 stationId = cfgView->getStationID();
 usa = cfgView->getUsa();
 offline = cfgView->getOnline();
 //cout << "Timeout: " << minutes << endl;
 //cout << "Station ID: " << stationId << endl;
 cfgView->hide();
 delete cfgView;
}

void Kweather::cancel()
{
 cfgView->hide();
 delete cfgView;
}
void Kweather::helpFile()
{
 KApplication::getKApplication()->invokeHTMLHelp("","");
}

/** This will display a dilog with the current weather. */
void Kweather::report(){
 	QString title;
 	QString message;
	QString tempText;
	int tempf;
	
	title = "Current Weather:";
	message = "Weather for ";
	message += stationId;
	message += "\nWind Speed: ";

	tempf = currentWeather.getWindSpeed();
	if (usa)
	{
	    tempText.setNum(tempf);
	    message += tempText;
	    message += " Knots";
	}
	else
	{
	    tempText.setNum((tempf * 185) / 100);
	    message += tempText;
	    message += " km/h";
	}

	tempf = currentWeather.getGustSpeed();
	if (tempf)
	{
	    message += "\nGust speed: ";

	    if (usa)
	    {
		tempText.setNum(tempf);
		message += tempText;
		message += " Knots";
	    }
	    else
	    {
		tempText.setNum((tempf * 185) / 100);
		message += tempText;
		message += " km/h";
	    }
	}

	message += "\nWind Direction: ";
	message += currentWeather.getWindDir();
	message += "\nVisibility: ";
	tempText.setNum(currentWeather.getVisability());
	message += tempText;

	message += " Statute Miles\nTemperature: ";
	tempf = currentWeather.getTemp();
	if (tempf == -100)	/* no temperature available */
	    message += "N/A";
	else
	{
	tempText.setNum(weather::toUS(usa, tempf));
	message += tempText;
	if ( !usa ) message += "\260C";
	else message += "\260F";
	}

	message += "\nDewpoint: ";
	tempf = currentWeather.getDewPoint();
	if (tempf == -100)	/* no dew available */
	    message += "N/A";
	else
	{
	tempText.setNum(weather::toUS(usa, currentWeather.getDewPoint()));
	message += tempText;
	if ( !usa ) message += "\260C";
	else message += "\260F";
	}
	
	message += "\nCurrent Weather: ";
	message += currentWeather.getCurrentWeather();
	message += "\nCloud Cover: ";
	message += currentWeather.getCover();
 	QMessageBox::information(this, title, message);
}

/** This will load the config file and set up the application. */
bool Kweather::loadConf(){
	QString configDir = KApplication:: localkdedir();
	QString configData;
	configDir += "/share/config/kweatherrc";
  //cout << configDir << endl;
 QFile f(configDir);
  if ( f.open(IO_ReadOnly) )
  {    // file opened successfully
     QTextStream t( &f );        // use a text stream

     while ( !t.eof() )
	{        // until end of file...
       		configData += t.readLine();       // line of text excluding '\n'     	
       	}
     	f.close();
			// Parse Config Data
			// Data will be in the following format
			// :usa:online:stationid:timeout:
			// :0:0:ABCD:000
			//cout << configData << endl;
      QString qsTimeout = configData.mid(10,3);
			QString qsId = configData.mid(5,4);
			QString qsOnline = configData.mid(3,1);
			QString qsUsa = configData.mid(1,1);
      minutes = qsTimeout.toInt();
			stationId = qsId;
			usa = qsUsa.toInt();
			offline = qsOnline.toInt();
			//cout << ":" << qsUsa << ":" << qsOnline << ":" << qsId << ":" << qsTimeout << ":" << endl;
			//cout << ":" << usa << ":" << offline << ":" << stationId << ":" << minutes << ":" << endl;
  }
	QRegExp r = QRegExp("[:0]", TRUE, FALSE);
	if ( stationId.contains(r) != 0 || stationId == "" || stationId == 0)
	{
		// set default values
			usa = false;
			offline = true;
			minutes = 10;
			cfgView = new configView(this, "config");
			cfgView->setTimeout( minutes);
 			cfgView->setStation( stationId );
			cfgView->setUsa( usa );
			cfgView->setOnline( offline );
			cfgView->show();
		
        }
	
	return (true);
}


